// import QtQuick 1.0 // to target S60 5th Edition or Maemo 5
import QtQuick 1.1
import Process 0.1

Item {
    id: root
    state: "idle"

    signal discoveringChanged
    signal hrDisconnecting
    signal hrDisconnectingComplete
    signal scanned
    signal connected
    signal hrConnectionFailed

    signal haveHrServiceHandle ( string notice )
    signal haveConfigHandle (string notice )
    signal haveBluetoothHrValue
    signal listening (string notice )
    property int heartrate: 0

    property bool scanning: false
    property bool scanHrHandle:false
    property bool readHR: false
    property bool waiting: false
    property string hr: "--"
    property string scriptsdir: "/usr/local/bin/truetouch/scripts"
    property int devicesFound: 0
    property bool deviceListChanged: false
    property bool connecting: false

    property string addrType: "random"
    property string deviceMAC: "NULL" //F6:94:92:22:03:5A" //ALPHA=F6:94:92:22:03:5A
    property string startHandle: "empty"
    property string hrHandle: "empty"
    property ListModel hrMonitors: monitorListModel
    property string hrUUID: "180d"
    property string configUUID: "2902"
  

    function myWait(delay){
        //var delay = 5; // 5 second delay
        var now = new Date();
        var desiredTime = new Date().setSeconds(now.getSeconds() + delay);

        while (now < desiredTime) {
            now = new Date(); // update the current time
        }
    }

    function scan() {
        // scan for heart rate monitors and return List
        root.scanning=true
        bleScan.start()

    }
    function buildArgs(deviceMAC,addrType) {
        var args = ["-b",deviceMAC, "-t",addrType]
        return args
    }

    function getHrServiceHandle(deviceMAC) {
        root.deviceMAC =deviceMAC
        var deviceArgs=buildArgs(root.deviceMAC,"random")

        var args=deviceArgs.concat(["--primary","--uuid="+root.hrUUID]);
        console.log(args)
        console.log("calling bleConnect.start process")
        bleGetHRServiceHandle.arguments = args
        root.scanning = true

        bleGetHRServiceHandle.start()
        console.log("finished bleConnect.start")
        console.log("startHandle is " + root.startHandle)

    }


    onHaveHrServiceHandle: {
        console.log(notice + "received onHave..")
        var skip=new RegExp("empty")
        if(!skip.test(root.startHandle)){
            var deviceArgs=buildArgs(root.deviceMAC,"random")
            var args=deviceArgs.concat(["--char-desc","-s",root.startHandle,root.hrUUID])
            console.log(args)
            console.log("calling bleGetCongigHandle.start process")
            myWait(3)
            bleGetConfigHandle.arguments =args
            root.scanHrHandle=true
            bleGetConfigHandle.start()


        }
    }

    onHaveConfigHandle:{
        console.log(notice + "received onHave..")
        console.log("handle is " + root.hrHandle)
        if(root.hrHandle!="empty"){
            var preArgs=["gatttool"]
            var deviceArgs=buildArgs(root.deviceMAC,"random")
            var args=preArgs.concat(deviceArgs,["--char-write-req","-a",root.hrHandle,"-n","0100","--listen"])
            bleReadHR.arguments=args
            root.waiting=false
            root.readHR=true
            myWait(3)
            bleReadHR.start()

        } else {
            console.log("StartHandle not found..now exiting.")
            root.waiting=false
        }
    }

    Process {
        id: bleGetConfigHandle
        program: "gatttool"

        onReadyReadStandardOutput: {
            console.log("processing standard output in get handle")
            var output = String(readAllStandardOutput())
            output = output.trim()
            console.log(output)
            var items = output.split(" ")
            var UUID=new RegExp(root.configUUID)
            // Now find handle that has UUID 2902

            for(var i=0;i<items.length;i++){
                console.log("item " + i+ "= \"" +items[i],"\"")
                //Note: This string handling
                // is different than on my PC
                console.log("checking handle: " + items[i])
                // if(items[i]==="2902\nhandle"){
                //if(items[i].match(/root.configUUID/)){
                if(UUID.test(items[i])){
                    root.hrHandle=items[i-3].split(",")[0]
                    console.log("uuid 2902 corresponds to handle " + root.hrHandle)
                    haveConfigHandle("emitted from bleGetConfig...")
                }
            }


        }

        onReadyReadStandardError: {
            console.log("processing standard error in get handle")
            var output = String(readAllStandardError())
            output = output.trim()
            console.log(output)
            
        }

        onFinished: {
            //root.scanHrHandle=false
            console.log("bleGetHrHandle finished")

        }


        function listargs(args){
            console.log("in listargs")
            console.log("args" + args)
        }
    }
    /*
    > gatttool -b $ALPHA --addr-type=random --char-write-req -a 0x22 -n 0100 --listen

        > Characteristic value was written successfully
        > Notification handle = 0x0021 value: 10 3d ef 03 
     */


    Process {
        id: bleReadHR
        // program: "gatttool"
        program: "unbuffer"

        onReadyReadStandardOutput: {
            console.log("processing standard output in bleReadHr")
            var output = String(readAllStandardOutput())
            var hr = "--"
            //output = output.trim()
            console.log(output)

            // Expected response
            // Notification handle = 0x0021 value: 10 5c 9c 02

            //  scanButtonLabel.text = "Connected"
            var items = output.split(" ")
            if(items[0]==="Notification"){

                hr=parseInt(items[6],16)
                //hr=items[6]
                console.log("Heartrate: "+hr)
                root.heartrate=hr
                haveBluetoothHrValue()
            }

            

        }

        onReadyReadStandardError: {
            console.log("processing standard error in bleReadHr")
            var output = String(readAllStandardError())
            output = output.trim()
            console.log(output)
        }

        onFinished: {
        }
    }




    Process {
        id: bleGetHRServiceHandle

        program: "gatttool"
        //program: "ls"
        //program: "gatttwrap.sh"


        onReadyReadStandardError: {
            console.log("processing standard error in bleGetHRServiceHandle")
            var output = String(readAllStandardError())
            output = output.trim()

        } // end onReadyReadStandardError:

        onReadyReadStandardOutput: {
            console.log("processing standard output in bleGetHRServiceHandle")
            var output = String(readAllStandardOutput())
            console.log( "scan output: " + output)
            output = output.trim()
            var handles = output.split(" ")

            for(var i=0 ; i < handles.length ; i++) {
                var handle=handles[i];
                console.log("handle " + i +" = " +handle)
            }
            root.startHandle="0x"+handles[2];


        } // end onReadyReadStandardOutput

        onFinished: {
            root.scanning = false
            console.log("startHandle "+root.startHandle)
            console.log("Running onFinished in bleGetHRServiceHandle")
            haveHrServiceHandle("emitted from bleGetHrServiceHandle\n")


        }
        function listargs(args){
            console.log("in listargs")
            console.log("args" + args)
        }


    }

    ListModel {
        id: monitorListModel
        ListElement{ MAC: "null"; name:"null"}
    }

    Process {
        id: bleScan

        program: "bash"
        arguments: [root.scriptsdir + "/start_smartScan.sh" ]


        onReadyReadStandardOutput: {
            var output= String(readAllStandardOutput())
            console.log(output)
            var item = output.split("\n")
            for(var i=0; i<item.length; i++){
                console.log("item " + i +"=" + item[i])
            }

            var MAC =[]
            var name=[]
            var current="empty"
            var skip=new RegExp("unknown")
            var j
            var duplicate=false
            for( i=0,j=0;i<item.length;i++){

                if((item[i]!=="LE Scan ...")&&(item[i]!=="")){
                    current=  item[i].split(" ")
                    console.log("current[1] = " + current[1])

                    MAC[j]=current[0]
                    name[j]=current[1]

                    //If the names is not "unknown" add to list
                    if(!skip.test(current[1])){
                        if(j===0){
                            console.log("adding [" + name[j] + "] to list")
                            hrMonitors.set(0,{"MAC":current[0],"name":current[1]})
                            j++
                        }
                        else {
                            duplicate=false
                            for(var  k=0; k< hrMonitors.count ; k++)
                                if(MAC[j]===hrMonitors.get(k).MAC){
                                    duplicate=true
                                    console.log("duplicate found skipping " + current[0] + current[1])
                                }
                            if(duplicate===false){
                                console.log("adding [" + name[j] + "] to list")
                                hrMonitors.append({"MAC":current[0],"name":current[1]})
                                console.log("List Length is " + hrMonitors.count)
                            }

                        }
                    }

                }
            }

            // Temporary needs to be generalized if more than one device is present
            // root.deviceMAC=MAC[0]
            //startMonitoring()
            for(j=0;j<hrMonitors.count;j++) {
                console.log("j=" +j)
                console.log(hrMonitors.get(j).name)
            }
        }

        onFinished: scanned()

    }//bleScan


    function hrDisconnect(deviceMAC,handle){
        console.log("Disconnect function launched")



        //Send signal to stop notification

        var deviceArgs=buildArgs(root.deviceMAC,"random")
        var args=deviceArgs.concat(["--char-write-req","-a",root.hrHandle,"-n","0000"])
        console.log(args)
        //Stop reading HR
        //bleReadHR.terminate()
        root.heartrate=NaN
        haveBluetoothHrValue()
        bleHRdisconnect.arguments=args
        bleHRdisconnect.start()
        //	root.hrHandle="empty"
    }

    Process {
        id: bleHRdisconnect

        program: "gatttool"
        //program: "ls"
        //program: "gatttwrap.sh"


        onReadyReadStandardError: {
            console.log("processing standard error in bleHRdisconnect")
            var output = String(readAllStandardError())
            output = output.trim()

        } // end onReadyReadStandardError:

        onReadyReadStandardOutput: {
            console.log("processing standard output in bleHRdisconnect")
            var output = String(readAllStandardOutput())
            console.log( "scan output: " + output)
            output = output.trim()



        } // end onReadyReadStandardOutput

        onFinished: {
            root.scanning = false

            console.log("Running onFinished in bleHRdisconnect")
            bleReadHR.terminate()
            hrDisconnectingComplete()


        }
        function listargs(args){
            console.log("in listargs")
            console.log("args" + args)
        }


    }


}

