import QtQuick 1.1
import Process 0.1
import Bluetooth 1.0

import ".."

Item {
    id: root
    Component.onCompleted: {

    }

    Component.onDestruction: {
        bluetoothAdapter.stopDiscovery()
    }



    Rectangle {
        id: bluetoothDeviceListBox

        color: "lightgray"
        radius: 10
        width: 400
        height: 510
        x: 120
        y: 20

        ListView {
            id: bluetoothDeviceList

            clip: true
            anchors.fill: parent
            anchors.margins: 5
            model: bluetoothAdapter.scannedDevices
            highlight: Rectangle { color: "darkgray" ; radius: 5 ; opacity: 0.7 ; width: 400 }
            highlightMoveSpeed: 1500
            delegate: Component {
                Item {
                    width: 400
                    height: 60
                    Column{

                        anchors { left: parent.left;top: parent.top; bottom: parent.bottom; right: parent.right }
                        Text {
                            text: name
                            font.pointSize: 16
                            color: paired ? "green" : "black"
                        }

                        Text {
                            text: address
                            font.pointSize: 16
                            color: paired ? "green" : "black"
                        }
                    }

                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            var item;
                            item = bluetoothDeviceList.model.itemAt(index);
                            bluetoothAdapter.connect(item)
                            bluetoothDeviceList.currentIndex = index
                            brainboard.buzzerButtonPress()
                        }
                    }

                }
            }
        }
    }


    ImageButton {
        id: scanButton
        x: 200
        y: 525

        enabled: !bluetoothAdapter.discovering && !bluetoothAdapter.connected
        visible: true
        label: qsTr("Scan") + application.emptyString
        label_color: "black"
        label_under: true
        upimage: "graphics/205/Lrg_Button_Up.png"
        downimage: "graphics/205/Lrg_Button_Down.png"
        onClicked: bluetoothAdapter.startDiscovery()
    }


    function pairDevice(index){
        var item;
        item = bluetoothDeviceList.model.get(index);
        bluetoothAdapter.createPairedDevice(item.address);
    }

    function removeDevice(index){
        var item;
        item = bluetoothDeviceList.model.get(index);
        bluetoothAdapter.removeDevice(item.address);
    }

    function connectAudio(index){
        var item;
        item = bluetoothDeviceList.model.itemAt(index);

        bluetoothAdapter.connect(item);
        //item.connectToAudioSource();
    }

    function disconnectAudio(index){
        var item;
        item = bluetoothDeviceList.model.itemAt(index);
        item.disconnectFromAudioSource();
    }



    ImageButton {
        id: removeButton

        enabled: !bluetoothAdapter.connected
        visible: true
        anchors { top: scanButton.top ; left: scanButton.right }
        label: qsTr("Unpair") + application.emptyString
        label_color: "black"
        label_under: true
        upimage: "graphics/205/Lrg_Button_Up.png"
        downimage: "graphics/205/Lrg_Button_Down.png"

        onClicked: removeDevice(bluetoothDeviceList.currentIndex)
    }

    ImageButton {
        id: disconnectAudioSourceButton

        enabled: bluetoothAdapter.connected
        visible: true
        anchors { top: scanButton.top ; left: removeButton.right }
        label: qsTr("disconnect") + application.emptyString
        label_color: "black"
        label_under: true
        upimage: "graphics/205/Lrg_Button_Up.png"
        downimage: "graphics/205/Lrg_Button_Down.png"

        onClicked: bluetoothAdapter.disconnect() //disconnectAudio(bluetoothDeviceList.currentIndex)
    }

    ImageButton {
        id: playButton

        enabled: bluetoothAdapter.connected
        visible: true
        anchors { top: scanButton.top ; left: disconnectAudioSourceButton.right }
        label: qsTr("play") + application.emptyString
        label_color: "black"
        label_under: true
        upimage: "graphics/205/Lrg_Button_Up.png"
        downimage: "graphics/205/Lrg_Button_Down.png"

        onClicked: {
            bluetoothAdapter.currentConnectedDevice.play();
        }
    }

    ImageButton {
        id: stopButton

        enabled: bluetoothAdapter.connected
        visible: true
        anchors { top: scanButton.top ; left: playButton.right }
        label: qsTr("stop") + application.emptyString
        label_color: "black"
        label_under: true
        upimage: "graphics/205/Lrg_Button_Up.png"
        downimage: "graphics/205/Lrg_Button_Down.png"

        onClicked: {
            bluetoothAdapter.currentConnectedDevice.stop();
        }
    }

    ImageButton {
        id: pauseButton

        enabled: bluetoothAdapter.connected
        visible: true
        anchors { top: scanButton.top ; left: stopButton.right }
        label: qsTr("pause") + application.emptyString
        label_color: "black"
        label_under: true
        upimage: "graphics/205/Lrg_Button_Up.png"
        downimage: "graphics/205/Lrg_Button_Down.png"

        onClicked: {
            bluetoothAdapter.currentConnectedDevice.pause();
        }
    }

    ImageButton {
        id: nextButton

        enabled: bluetoothAdapter.connected
        visible: true
        anchors { top: scanButton.top ; left: pauseButton.right }
        label: qsTr("next") + application.emptyString
        label_color: "black"
        label_under: true
        upimage: "graphics/205/Lrg_Button_Up.png"
        downimage: "graphics/205/Lrg_Button_Down.png"

        onClicked: {
            bluetoothAdapter.currentConnectedDevice.next();
        }
    }

    ImageButton {
        id: previousButton

        enabled: bluetoothAdapter.connected ? true : false
        visible: true
        anchors { top: scanButton.top ; left: nextButton.right }
        label: qsTr("previous") + application.emptyString
        label_color: "black"
        label_under: true
        upimage: "graphics/205/Lrg_Button_Up.png"
        downimage: "graphics/205/Lrg_Button_Down.png"

        onClicked: {
            bluetoothAdapter.currentConnectedDevice.previous();
        }
    }

    Connections{
        target: bluetoothAdapter
        onPasskeyAvailable:{
            passkeyDialog.text = passkey
            passkeyDialog.show()
        }
        onPairingComplete:{
            passkeyDialog.hide()
        }
    }

    ModalDialog {
        id: passkeyDialog
        visible: false  // shown when update process starts
        property string text: ""

        BorderImage {   // pop-up dialog background
            id: border

            x: 332
            y: 161
            width: 800
            height: 350
            source: "../graphics/Pop_Up_Lrg_Grey.png"
            border { top: 152; bottom: 165; left: 145; right: 144 }

            Text {
                id: dialogText
                x: 149
                y: 35
                width: 750
                height: 300
                color: "#ffffff"
                text: "Bluetooth Pairing Request Code:\r\n" + paddy(passkeyDialog.text,6)
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                //anchors.topMargin: 35
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.bold: true
                font.pixelSize: 60
            }
        }
    }

    function paddy(n, p, c){
        var pad_char = typeof c !== 'undefined' ? c : '0';
        var pad = new Array(1+p).join(pad_char);
        return (pad+n).slice(-pad.length);
    }
}



