
import QtQuick 1.1
import VLCVideo 1.0

import ".."

Image {

    id: root

    property bool showCloseButton: true
    property int highlightedItem:1
    property bool isDialogVisible: false
    property variant menu
    property int menu_bottom_margin: 70
    property bool videoShown: false
    property bool viewShowing: false
    property bool keysAccepted: false
    property bool allowFullscreen: false

    state: ipodmanager.browserState

    Connections {
        target: !main.blockKeys && main.isTft && keysAccepted && root.opacity > 0 && !videoView.visible ? keymapper : null

        onBackPressed: {
            main.debugKeys("backPressed", "MediaBrowser")
            close()
        }
    }

    // iPod - Menu Navigation
    Connections {
        target: main.isTft && root.opacity > 0 && keysAccepted && (state === "notplaying_menu" || state === "playing_menu") ? keymapper : null

        onDoubleLeftPressed: {
            main.debugKeys("doubleLeftPressed", "MediaBrowser")
            leftPressed()
        }
        onDoubleRightPressed: {
            main.debugKeys("doubleRightPressed", "MediaBrowser")
            rightPressed()
        }
        onUpButtonPressed: {
            main.debugKeys("upButtonPressed", "MediaBrowser")
            ipodmenulistview.decrementCurrentIndex()
            upPressed()
        }
        onDownButtonPressed: {
            main.debugKeys("downButtonPressed", "MediaBrowser")
            ipodmenulistview.incrementCurrentIndex()
            downPressed()
        }
        onEnter2Pressed: {
            main.debugKeys("enter2Pressed", "MediaBrowser")
            centerPressed()
        }
    }

    // Video Controls - key controls when video is playing
    Connections {
        target: main.isTft && root.opacity > 0 && keysAccepted && videoShown && (state === "playing_notmenu" || state === "notplaying_notmenu") ? keymapper : null

        onEnter2Pressed: {
            main.debugKeys("enter2Pressed", "MediaBrowser", "video shown")
            root.toggleFullscreen()
        }
        onDoubleLeftPressed: {
            main.debugKeys("doubleLeftPressed", "MediaBrowser", "video shown")
            leftPressed()
        }
        onDoubleRightPressed: {
            main.debugKeys("doubleRightPressed", "MediaBrowser", "video shown")
            rightPressed()
        }

        onBackPressed: {
            main.debugKeys("backPressed", "MediaBrowser", "video shown")
            if (tv.player.fullscreen)
                return
            else
                menuPressed()
        }

        onLeftButtonPressed: {
            main.debugKeys("leftButtonPressed", "MediaBrowser", "video shown")
            if (tv.player.fullscreen)
                root.toggleFullscreen()
        }

        onRightButtonPressed: {
            main.debugKeys("rightButtonPressed", "MediaBrowser", "video shown")
            if (tv.player.fullscreen)
                root.toggleFullscreen()
        }

        onUpButtonPressed: {
            main.debugKeys("upButtonPressed", "MediaBrowser", "video shown")
            volume_controls.volumeUp()
        }
        onDownButtonPressed: {
            main.debugKeys("downButtonPressed", "MediaBrowser", "video shown")
            volume_controls.volumeDown()
        }
    }

    // iPod - Playing controls - Excludes video
    Connections {
        target: main.isTft && root.opacity > 0 && keysAccepted && !videoShown && (state === "playing_notmenu" || state === "notplaying_notmenu") ? keymapper : null

        onEnter2Pressed: {
            main.debugKeys("enter2Pressed", "MediaBrowser", "not menu")
            menuPressed()
        }
        onDoubleLeftPressed: {
            main.debugKeys("doubleLeftPressed", "MediaBrowser")
            leftPressed()
        }
        onDoubleRightPressed: {
            main.debugKeys("doubleRightPressed", "MediaBrowser", "not menu")
            rightPressed()
        }
        onUpButtonPressed: {
            main.debugKeys("upButtonPressed", "MediaBrowser", "not menu")
            volume_controls.volumeUp()
        }
        onDownButtonPressed: {
            main.debugKeys("downButtonPressed", "MediaBrowser", "not menu")
            volume_controls.volumeDown()
        }
    }

    function previousTrack() {
    }

    function resetMenuState(){
        ipodmanager.clear();
    }

    function toggleFullscreen() {
        if (root.allowFullscreen) {
            if (tv.player.fullscreen) {
                fullscreenVideoFrame.endFullscreen()
            } else {
                fullscreenVideoFrame.showFullscreen(tv.player)
            }
        }
    }

    function showVideo(show) {
        if(show) {
            tv.tuneChannelID("IPODINPUT")
            root.videoShown = true
        }
        else
            root.videoShown = false
    }

    function shown() {
        viewShowing = true
        if(audioManager.ipodPlaying)
            audioManager.ipodRequest()
        if(audioManager.ipodPlayingVideo) {
            // if an ipod video is already allowed and we're not showing it, show it.
            if(audioManager.ipodVideoAllowed) {
                root.showVideo(true)
            }
            else // request showing the videov
                audioManager.ipodVideoRequest()
        }
        if (audioManager.btconnected) {
            audioManager.btRequest()
        }
    }

    function hidden() {
        viewShowing = false
    }

    signal close()

    signal upPressed()
    signal downPressed()
    signal rightPressed()
    signal leftPressed()
    signal centerPressed()
    signal menuPressed()
    signal menuItemPressed(int index)
    signal playPausePressed()
    signal nextTrackPressed()
    signal previousTrackPressed()

    source: "../graphics/netpulse/Center_Gray_Box.png"


    function changeState(event)
    {
        //console.debug("changing state - State: "+root.state+" Event: "+event)
        switch(root.state)
        {
        case "notplaying_notmenu":
            switch(event){
            case "centerbuttonpressed":
                brainboard.iPodPlayPause()
                break;
            case "upbuttonpressed": break;
            case "downbuttonpressed": break;
            case "leftbuttonpressed":
                brainboard.iPodPreviousTrack()
                break;
            case "rightbuttonpressed":
                brainboard.iPodNextTrack()
                break;
            case "menupressed":
                ipodmanager.browserState = "notplaying_menu"
                break;
            case "timertriggered":
                break;
            case "pausestop":
                break;
            case "playing": {
                ipodmanager.browserState = "playing_notmenu"
                ipodTimer.stop()
                break;
            }
            } // end switch(event)
            break;
        case "notplaying_menu":
            switch(event){
            case "centerbuttonpressed":
                menuItemPressed(ipodmenulistview.currentIndex)
                break;
            case "upbuttonpressed": break;
            case "downbuttonpressed": break;
            case "leftbuttonpressed": {
                ipodmanager.leftButtonPressedSlot()
                break;
            }
            case "rightbuttonpressed":
                menuItemPressed(ipodmenulistview.currentIndex)
                break;
            case "menupressed": {
                ipodmanager.leftButtonPressedSlot()
                break;
            }
            case "timertriggered":
                break;
            case "pausestop":
                break;
            case "playing": {
                ipodmanager.browserState = "playing_notmenu"
                ipodTimer.stop()
                break;
            }

            } //end switch(event)
            break;
        case "playing_notmenu":
            switch(event){
            case "centerbuttonpressed":
                brainboard.iPodPlayPause()
                break;
            case "upbuttonpressed": break;
            case "downbuttonpressed": break;
            case "leftbuttonpressed":
                brainboard.iPodPreviousTrack()
                break;
            case "rightbuttonpressed":
                brainboard.iPodNextTrack()
                break;
            case "menupressed": {
                ipodmanager.browserState = "playing_menu"
                ipodTimer.start()
                break;
            }
            case "timertriggered":
                break;
            case "pausestop":
                ipodmanager.browserState = "notplaying_notmenu"
                break;
            case "playing":
                break;
            }
            break;
        case "playing_menu":
            switch(event){
            case "centerbuttonpressed": {
                ipodTimer.restart()
                menuItemPressed(ipodmenulistview.currentIndex)
                break;
            }
            case "upbuttonpressed": {
                ipodTimer.restart()
                break;
            }
            case "downbuttonpressed": {
                ipodTimer.restart()
                break;
            }
            case "leftbuttonpressed": {
                ipodTimer.restart()
                ipodmanager.leftButtonPressedSlot()
                break;
            }
            case "rightbuttonpressed": {
                ipodTimer.restart()
                print("right button current index:" + ipodmenulistview.currentIndex)
                menuItemPressed(ipodmenulistview.currentIndex)
                break;
            }
            case "menupressed": {
                ipodTimer.restart()
                ipodmanager.leftButtonPressedSlot()
                break;
            }
            case "timertriggered": {
                ipodmanager.browserState = "playing_notmenu"
                ipodTimer.stop()
                break;
            }
            case "pausestop": {
                ipodmanager.browserState = "notplaying_menu"
                ipodTimer.stop()
                break;
            }
            case "playing":
                break;
            case "trackchanged": {
                ipodmanager.browserState = "playing_notmenu"
                ipodTimer.stop()
                break;
            }
            }
            break;
        }
    }

    states:[
        State{name: "not_connected"
            PropertyChanges {target: now_playing; visible: false}
            PropertyChanges {target: ipodmenu; visible: false}
            //PropertyChanges {target: videoPlayer; visible: false}
            PropertyChanges {target: ipod_wheel; visible: false}
            PropertyChanges {target: menu_button; visible: false}
            PropertyChanges {target: nike_plus_button; visible: false}
        },

        State{name: "notplaying_notmenu"
            PropertyChanges {target: now_playing; visible: true}
            PropertyChanges {target: ipodmenu; visible: false}
            PropertyChanges {target: ipod_wheel; visible: main.isTouch }
            PropertyChanges {target: help_pane; visible: main.isTft }
            PropertyChanges {target: menu_button; visible: main.isTouch }
            PropertyChanges {target: nike_plus_button; visible: ipod_wheel.visible & ipodmanager.nikePlusSupport}
        },
        State{name: "notplaying_menu"
            PropertyChanges {target: now_playing; visible: false}
            PropertyChanges {target: ipodmenu; visible: true}
            PropertyChanges {target: ipod_wheel; visible: main.isTouch }
            PropertyChanges {target: help_pane; visible: main.isTft }
            PropertyChanges {target: menu_button; visible: main.isTouch }
            PropertyChanges {target: nike_plus_button; visible: ipod_wheel.visible & ipodmanager.nikePlusSupport}

        },
        State{name: "playing_notmenu"
            PropertyChanges {target: now_playing; visible: !root.videoShown}
            PropertyChanges {target: ipodmenu; visible: false}
            PropertyChanges {target: ipod_wheel; visible: main.isTouch }
            PropertyChanges {target: help_pane; visible: main.isTft }
            PropertyChanges {target: menu_button; visible: main.isTouch }
            PropertyChanges {target: nike_plus_button; visible: ipod_wheel.visible & ipodmanager.nikePlusSupport}

        },
        State{name: "playing_menu"
            PropertyChanges {target: now_playing; visible: false}
            PropertyChanges {target: ipodmenu; visible: true}
            PropertyChanges {target: ipod_wheel; visible: main.isTouch }
            PropertyChanges {target: help_pane; visible: main.isTft }
            PropertyChanges {target: menu_button; visible: main.isTouch }
            PropertyChanges {target: nike_plus_button; visible: ipod_wheel.visible & ipodmanager.nikePlusSupport}

        }
    ]

    Timer{
        id:ipodTimer
        interval: 10000
        running: false
        onTriggered: {
            //console.debug("Timer triggered")
            changeState("timertriggered")
        }
    }

    Connections{
        target: root
        onUpPressed: {console.debug("UP");changeState("upbuttonpressed")}
        onDownPressed: {console.debug("DOWN");changeState("downbuttonpressed")}
        onRightPressed:{ console.debug("RIGHT");changeState("rightbuttonpressed")}
        onLeftPressed: {changeState("leftbuttonpressed")}
        onCenterPressed:{ console.debug("CENTER");changeState("centerbuttonpressed")}
        onMenuPressed: {changeState("menupressed")}
        onMenuItemPressed:{ ipodmanager.menuItemPressedSlot(index)}
    }

    Connections{
        target: brainboard
        onIPodTrackChanged: {changeState("trackchanged"); console.debug("trackchanged")}
        onIPodPlayingStatusChanged: {
            if(brainboard.iPodPlayingStatus === "playing") {
                changeState("playing")
            }
            else
            {
                changeState("pausestop")
            }
        }
    }

    // mouse area to mask anything under this pop-up
    MouseArea {
        anchors.fill: parent
    }

    function getIndexFromMenuItem(i)
    {
        var item = ipodmenulistview.model.get(i)
        return item.index
    }

    MouseArea{
        id: cover_area
        visible: connectionDialog.visible
        z: 99
        anchors.fill: parent
        anchors.rightMargin: 70
    }

    ModalDialog {
        id: connectionDialog
        opacity: main.isTouch || root.keysAccepted ? 1.0 : 0.5
        isTop: false
        visible: (!ipodmanager.iPodConnected &&
                  !bluetoothAdapter.connected &&
                  !btStatusDialog.visible)

        anchors.verticalCenter: parent.verticalCenter
        anchors.horizontalCenter: parent.horizontalCenter

        Connections {
            target: main.isTft && !main.blockKeys && root.keysAccepted && connectionDialog.visible ? keymapper : null
            onEnter2Pressed: {
                main.debugKeys("enter2Pressed", "MediaBrowser", "(connectionDialog)")
                if (!bluetoothAdapter.discovering)
                    bluetoothAdapter.startDiscovery()
            }
        }



        BorderImage {   // pop-up dialog background
            id: border
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            width: root.width * 0.8
            height: width / 2
            source: "../graphics/Pop_Up_Lrg_Grey.png"
            border { top: height*0.33; bottom: height*0.33; left: width*0.33; right: width*0.33 }

            Text {
                id: dialogText
                anchors.verticalCenter: parent.verticalCenter
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenterOffset: -60
                width: parent.width * 0.8
                height: parent.height * 0.5
                color: "white"
                text: qsTr("NO APPLE DEVICE DETECTED") + application.emptyString
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.bold: true
                font.family: "sans serif"
                font.pixelSize: ["ru","nl","es","pt","it"].indexOf(application.currentLanguage) >= 0 ? 50 : 62
            }
            Item {
                id: btConnectionPromptBox
                anchors.verticalCenter: parent.verticalCenter
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenterOffset: 70
                width: childrenRect.width
                height: childrenRect.height
                Column{
                    //anchors.fill: parent
                    Text{
                        id: btAudioDeviceText
                        anchors.horizontalCenter: parent.horizontalCenter
                        text: qsTr("BLUETOOTH AUDIO DEVICE?") + application.emptyString
                        font.bold: true
                        font.family: "sans serif"
                        font.pixelSize: dialogText.font.pixelSize * 0.3
                        color: "white"
                    }
                    ImageButton {
                        id: btScanButton
                        visible: main.isTouch
                        anchors.horizontalCenter: parent.horizontalCenter
                        upimage: "graphics/224/224_Data_ToolBar_Round.png"
                        downimage: "graphics/224/224_Data_ToolBar_Round_Down.png"
                        icon: "graphics/Bluetooth/Bluetooth_icon_small.png"
                        onClicked: {
                            if(bluetoothAdapter.discovering == true){
                                bluetoothAdapter.stopDiscovery()
                            }

                            bluetoothAdapter.startDiscovery()

                        }
                    }
                    Item {
                        id: btScanTextItem
                        visible: main.isTft
                        anchors.horizontalCenter: parent.horizontalCenter
                        width:childrenRect.width
                        height:childrenRect.height
                        Image {
                            id:btTextImage
                            source: "../graphics/Bluetooth/Bluetooth_icon_small.png"
                            anchors.verticalCenter: parent.verticalCenter
                            anchors.left:parent.left
                        }
                        Text {
                            id: btScanTextTft
                            anchors.verticalCenter: parent.verticalCenter
                            anchors.left: btTextImage.right
                            anchors.leftMargin: 10
                            text: qsTr("press ENTER to search for devices") + application.emptyString
                            font.bold: true
                            font.family: "sans serif"
                            font.pixelSize: 24
//                            color: "#005dab"
                            color: "#ffffff"
                        }
                    }

                    Text{
                        id: btAudioDeviceMustBeDiscoverableText
                        anchors.horizontalCenter: parent.horizontalCenter
                        text: qsTr("DEVICE MUST BE DISCOVERABLE") + application.emptyString
                        font.bold: true
                        font.family: "sans serif"
                        font.pixelSize: dialogText.font.pixelSize * 0.3
                        color: "white"
                    }
                }
            }
            ImageButton{
                id: btHelpButton
                visible: main.isTouch
                upimage: "graphics/netpulse/GRAYbutton_video.png"
                downimage: "graphics/netpulse/GRAYbutton_video_PRESSED.png"
                anchors.bottom: btConnectionPromptBox.bottom
                anchors.left: btConnectionPromptBox.right
                width: 50
                height: 50
                onClicked: {
                    btHelpDialog.show()
                }
                label: "?"
                label_under: false
                label_color: "black"
            }
        }


    }


    ModalDialog {
        id: btHelpDialog
        isTop: false
        isTransparent: true
        visible: false
        z: connectionDialog.z + 1

        anchors.verticalCenter: parent.verticalCenter
        anchors.horizontalCenter: parent.horizontalCenter

        Connections {
            target: main.isTft && !main.blockKeys && root.keysAccepted && btHelpDialog.visible ? keymapper : null
            onBackPressed: {
                main.debugKeys("backPressed", "MediaBrowser", "(btHelpDialog)")
                helpExitButton.clicked()
            }
        }

        BorderImage {   // pop-up dialog background
            id: border1
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            width: root.width * 0.8
            height: width / 2
            source: "../graphics/Pop_Up_Lrg_Grey.png"
            border { top: height*0.33; bottom: height*0.33; left: width*0.33; right: width*0.33 }

            Item {
                id: helpTextItems
                anchors.verticalCenter: parent.verticalCenter
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.fill: parent
                ImageButton{
                    id: helpExitButton
                    visible: main.isTouch
                    upimage: "Keypad/KeyPad_X.png"
                    downimage: "Keypad/KeyPad_X.png"
                    anchors.top: parent.top
                    anchors.topMargin: 70
                    anchors.right: parent.right
                    anchors.rightMargin: 70
                    onClicked: btHelpDialog.hide()
                }
                Text{
                    id: fromYourDeviceText
                    anchors.top: parent.top
                    anchors.topMargin: 30
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width
                    text: qsTr("FROM YOUR DEVICE:") + application.emptyString
                    color: "white"
                    horizontalAlignment: Text.AlignHCenter
                    wrapMode: Text.WordWrap
                    font.bold: true
                    font.family: "sans serif"
                    font.pixelSize: parent.height * 0.12
                }
                Column{
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.top: fromYourDeviceText.bottom
                    anchors.topMargin: 20
                    width: parent.width * 0.8
                    spacing: 20
                    Text{
                        id: appleDeviceText
                        width: parent.width
                        text: qsTr("- APPLE DEVICE: go to Settings, then Bluetooth") + application.emptyString
                        color: "white"
                        horizontalAlignment: Text.AlignLeft
                        wrapMode: Text.WordWrap
                        font.bold: true
                        font.family: "sans serif"
                        font.pixelSize: fromYourDeviceText.height * 0.4
                    }
                    Text{
                        id: androidDeviceText
                        width: parent.width
                        text: qsTr("- ANDROID DEVICE: go to Bluetooth Settings") + application.emptyString
                        color: "white"
                        horizontalAlignment: Text.AlignLeft
                        wrapMode: Text.WordWrap
                        font.bold: true
                        font.family: "sans serif"
                        font.pixelSize: fromYourDeviceText.height * 0.4
                    }
                    Text{
                        id: confirmConnectionText
                        width: parent.width
                        text: qsTr("- Confirm connection from your device") + application.emptyString
                        color: "white"
                        horizontalAlignment: Text.AlignLeft
                        wrapMode: Text.WordWrap
                        font.bold: true
                        font.family: "sans serif"
                        font.pixelSize: fromYourDeviceText.height * 0.4
                    }
                    Text{
                        id: navigateAndPlayText
                        width: parent.width
                        text: qsTr("- NAVIGATE and PLAY content from your device, connect ear buds and control volume from this console.") + application.emptyString
                        color: "white"
                        horizontalAlignment: Text.AlignLeft
                        wrapMode: Text.WordWrap
                        font.bold: true
                        font.family: "sans serif"
                        font.pixelSize: fromYourDeviceText.height * 0.4
                    }
                }
            }
        }
        onTimedOut: hide()
    }

    Item {
        id: ipod_wheel
        x: 562

        visible: true
        width: 300
        height: 300
        anchors.top: parent.top
        anchors.topMargin: 86
        ImageButton {
            id: up_button
            x: 43
            y: -93
            anchors.bottom: center_button.verticalCenter
            anchors.bottomMargin: 42
            anchors.horizontalCenterOffset: 0
            anchors.horizontalCenter: center_button.horizontalCenter
            upimage: "graphics/205/IPOD_Wheel_Top_Up.png"
            downimage: "graphics/205/IPOD_Wheel_Top_Down.png"
            icon: "graphics/205/IPOD_Controls_Up.png"
            onClicked:
            {
                if(ipodmenulistview.currentIndex > 0)
                {
                    ipodmenulistview.currentIndex--
                }
                upPressed()
            }
            mouseAreaScale: 0.7

        }
        ImageButton {
            id: center_button
            x: 114
            y: 66

            upimage: (ipodmanager.playStatus == "playing") ? downimage : "graphics/205/IPOD_Wheel_Center_Up.png"
            downimage: "graphics/205/IPOD_Wheel_Center_Down.png"
            icon: "graphics/205/IPOD_Controls_Select.png"

            onClicked: {
                centerPressed()
            }
            mouseAreaScale: 0.7
        }

        ImageButton {
            id: down_button
            anchors.horizontalCenterOffset: 1
            anchors.top: center_button.verticalCenter
            anchors.topMargin: 42
            anchors.horizontalCenter: center_button.horizontalCenter
            upimage: "graphics/205/IPOD_Wheel_Bottom_Up.png"
            downimage: "graphics/205/IPOD_Wheel_Bottom_Down.png"
            icon: "graphics/205/IPOD_Controls_Down.png"
            onClicked:
            {
                if(ipodmenulistview.currentIndex + 1 < ipodmenulistview.count)
                    ipodmenulistview.currentIndex++
                downPressed()
            }
            mouseAreaScale: 0.7

        }

        ImageButton {
            id: left_button
            x: 0
            y: 116
            anchors.right: center_button.horizontalCenter
            anchors.rightMargin: 36
            anchors.verticalCenterOffset: 0
            anchors.verticalCenter: center_button.verticalCenter
            anchors.topMargin: 0
            upimage: "graphics/205/IPOD_Wheel_Left_Up.png"
            downimage: "graphics/205/IPOD_Wheel_Left_Down.png"
            icon: "graphics/205/IPOD_Controls_Back.png"
            onClicked: {
                leftPressed()
            }
            mouseAreaScale: 0.7

        }

        ImageButton {
            id: right_button
            anchors.verticalCenterOffset: 0
            anchors.left: center_button.horizontalCenter
            anchors.leftMargin: 41
            anchors.verticalCenter: center_button.verticalCenter
            upimage: "graphics/205/IPOD_Wheel_Right_Up.png"
            downimage: "graphics/205/IPOD_Wheel_Right_Down.png"
            icon: "graphics/205/IPOD_Controls_Forward.png"
            onClicked: {
                rightPressed()
            }
            mouseAreaScale: 0.7

        }

        Rectangle {
            id: wheel_well
            // helpful for alignment of circular button array, as buttons have shadows
            x: 75
            y: 43
            width: 285
            height: width
            color: "#ffffff"
            radius: width*0.5
            anchors.verticalCenterOffset: -2
            anchors.horizontalCenterOffset: -2
            z: -1
            anchors.horizontalCenter: center_button.horizontalCenter
            anchors.verticalCenter: center_button.verticalCenter
            opacity: 0
        }
    } // end ipod wheel

    IPodTftHelp {
        id: help_pane
        ipodState: root.state
        keysAccepted: root.keysAccepted
        allowFullscreen: root.allowFullscreen
        x: 532
        anchors.top: parent.top
    }

    NowPlayingControl {
        id: now_playing
        opacity: main.isTouch || root.keysAccepted ? 1.0 : 0.5

        x: 31
        y: 20
        //visible: !isMenuVisible
        // connect data model for song meta-data here (duration, title, etc.)

        duration: ipodmanager.nowPlayingTrackDuration
        //title: title
        title: ipodmanager.nowPlayingTrackName
        artist: ipodmanager.nowPlayingTrackArtist
        album: ipodmanager.nowPlayingTrackAlbum
        position: ipodmanager.nowPlayingTrackTimeElapsed

        visible: true
        // scrub event happens when user moves slider
        // onScrub: { now_playing.position = new_time }

        anchors.top: parent.top
        anchors.topMargin: 36
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 80
        anchors.left: parent.left
        anchors.leftMargin: 31


    }

    ImageButton{
        id: nike_plus_button
        anchors {
            top: ipod_wheel.bottom ; topMargin: -50
            horizontalCenter: ipod_wheel.horizontalCenter
        }
        upimage:"graphics/205/SM_Button_Up.png"
        downimage:"graphics/205/SM_Button_Down.png"
        icon: ipodmanager.nikePlusRecord ? "graphics/205/IPOD_Checkmark.png" : ""
        label: "Nike + iPod"
        fontsize: 30
        label_color: "white"
        label_x_offset: 120
        onClicked: {

            ipodmanager.nikePlusRecord = !ipodmanager.nikePlusRecord

        }

        enabled: main.state == "RunState" & ipodmanager.nikePlusRecord ? false : true
    }

    Connections{
        target:ipodmanager
        onIPodConnectedChanged: {
            resetMenuState()
            if(ipodmanager.iPodConnected){
                if(bluetoothAdapter.connected){
                    bluetoothAdapter.disconnect()
                }
                btStatusDialog.state = "idle"
                btStatusDialog.visible = false
                btHelpDialog.hide()
            }
        }

    }



    IPodMenuControl{

        id:ipodmenu
        property string selectedItemHeading: ""
        opacity: main.isTouch || root.keysAccepted ? 1.0 : 0.5
        x: 31
        y: 20
        //visible: isMenuVisible
        anchors.top: parent.top
        anchors.topMargin: 36
        anchors.bottom: parent.bottom
        anchors.bottomMargin: root.menu_bottom_margin
        anchors.left: parent.left
        anchors.leftMargin: 31
        visible: true

        heading: ipodmanager.menuHeading


        ListView {
            id: ipodmenulistview

            anchors.fill: parent
            anchors.topMargin: 80
            anchors.bottomMargin: 20
            anchors.leftMargin: 30
            anchors.rightMargin: 20
            clip: true
            onMovingChanged: root.upPressed()
            currentIndex: 1
            highlight: Rectangle { color: "lightsteelblue"; radius: 1 }
            pressDelay: 200
            highlightMoveDuration: 1
            //model: testmodel
            model: ipodmanager.menu
            visible: true
            delegate: Component {
                Row {
                    Image {
                        source: "../graphics/tft/toggle_arrow_black.png"
                        visible: label !== "" && (["Tracks", "Music Videos", "TV Shows", "Video Podcasts", "Video iTunes U", "Audiobooks"].indexOf(ipodmanager.menuHeading) < 0)
                    }
                    Text {
                        text: label
                        width: ipodmenulistview.width
                        height: label == "" ? 0 : implicitHeight
                        MouseArea{
                            anchors.fill: parent
                            onClicked: {
                                menuItemPressed(index)
                            }
                            onPressed: {
                                ipodmenulistview.currentIndex = index
                                brainboard.buzzerButtonPress()
                            }
                        }
                    }
                }
            }
        }
    }

    // close button
    ImageButton {
        id: close_button

        z: 200

        visible: root.showCloseButton && main.isTouch
        width: 48
        height: 48
        anchors {   right: parent.right ; rightMargin: 40 ;
            top: parent.top ; topMargin: 40
        }
        upimage: "Keypad/KeyPad_X_Large.png"
        downimage: "Keypad/KeyPad_X_Large.png"

        onClicked: close()
        mouseAreaScale: 2.0

    }

    ImageButton {
        id: menu_button
        anchors.top: ipod_wheel.bottom
        anchors.topMargin: -285
        anchors.left: ipod_wheel.right
        anchors.leftMargin: 30
        fontsize: 14
        label: qsTr("MENU") + application.emptyString
        upimage: "graphics/205/Lrg_Button_Up.png"
        downimage: "graphics/205/Lrg_Button_Down.png"
        onClicked: {
            menuPressed()
        }
    }

    VLCSurface {
        id: videoView

        player: tv.player
        playing: !player.fullscreen && visible && ( tv.playing  || tv.tuning)
        aspect: tv.aspect

        x: 31
        width: 545
        height: width / tv.aspect
        anchors.verticalCenter: parent.verticalCenter

        visible: root.videoShown && root.state == "playing_notmenu"

        MouseArea {
            anchors.fill: parent
            onClicked: fullscreenVideoFrame.showFullscreen(tv.player)
        }
    } // end videoView

    Connections {
        target: root.viewShowing ? audioManager : null

        ignoreUnknownSignals: true

        onIpodVideoAllowedChanged: {
            if(audioManager.ipodVideoAllowed){
                if(audioManager.ipodPlayingVideo && viewShowing){
                    showVideo(true)
                }
            }
            else {
                showVideo(false)
            }
        }

        onIpodPlayingChanged: {
            if(audioManager.ipodPlaying && viewShowing){
                audioManager.ipodRequest()
                if(audioManager.ipodPlayingVideo && audioManager.ipodVideoAllowed) {
                    showVideo(true)
                }
            }
        }

        onBtconnectedChanged: {
            if (audioManager.btconnected && viewShowing) {
                audioManager.btRequest()
            }
        }

        onIpodPlayingVideoChanged: {
            if(audioManager.ipodPlayingVideo && viewShowing){
                if(!videoShown) {
                    if(audioManager.ipodVideoAllowed) {
                        showVideo(true)
                    }
                    else {  // ask for permission to show the video
                        audioManager.ipodVideoRequest()
                    }
                }
            }
            else if(!audioManager.ipodPlayingVideo) {
                showVideo(false)
            }
        }
    }

    NowPlayingControl {
        id: bt_now_playing
        opacity: main.isTouch || root.keysAccepted ? 1.0 : 0.5

        x: 31
        y: 20
        //visible: !isMenuVisible
        // connect data model for song meta-data here (duration, title, etc.)
        scrubControlVisible: false
        duration: 2
        //title: title
        title: bluetoothAdapter.currentConnectedDevice ? bluetoothAdapter.currentConnectedDevice.currentTitle : ""
        artist: bluetoothAdapter.currentConnectedDevice ? bluetoothAdapter.currentConnectedDevice.currentArtist : ""
        album: bluetoothAdapter.currentConnectedDevice ? bluetoothAdapter.currentConnectedDevice.currentAlbum : ""
        position: 1

        visible: bluetoothAdapter.connected
        // scrub event happens when user moves slider
        // onScrub: { now_playing.position = new_time }

        anchors.top: parent.top
        anchors.topMargin: 36
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 80
        anchors.left: parent.left
        anchors.leftMargin: 31


    }

    ImageButton {
        id: bt_menu_button
        visible: bluetoothAdapter.connected && main.isTouch
        anchors.top: bluetooth_controls.bottom
        anchors.topMargin: -285
        anchors.left: bluetooth_controls.right
        anchors.leftMargin: 30
        fontsize: 14
        label: qsTr("MENU") + application.emptyString
        upimage: "graphics/205/Lrg_Button_Up.png"
        downimage: "graphics/205/Lrg_Button_Down.png"
        onClicked: {
            menuPressed()
        }
        enabled: false
    }

    ImageButton{
        id: bt_disconnect_button
        visible: main.isTouch && bluetoothAdapter.connected
        upimage: "graphics/205/Lrg_Button_Up.png"
        downimage: "graphics/205/Lrg_Button_Down.png"
        icon: "graphics/Bluetooth/Bluetooth_icon_small.png"
        icon_x_offset: -5
        icon_y_offset: 0
        anchors.horizontalCenter: bt_menu_button.horizontalCenter
        anchors.verticalCenter: bt_nike_plus_button.verticalCenter
        onClicked: if(bluetoothAdapter.connected) bluetoothAdapter.disconnect()
        Image{
            source: "../graphics/Bluetooth/small_red_X.png"
            width: 18
            height: 17
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.horizontalCenterOffset: 8
            anchors.verticalCenter: parent.verticalCenter
            anchors.verticalCenterOffset: 10
            fillMode: "PreserveAspectFit"
            smooth: true
        }
    }

    ImageButton{
        id: bt_nike_plus_button
        anchors {
            top: bluetooth_controls.bottom ; topMargin: -50
            horizontalCenter: bluetooth_controls.horizontalCenter
        }
        upimage:"graphics/205/SM_Button_Up.png"
        downimage:"graphics/205/SM_Button_Down.png"
        icon: ""
        label: "Nike + iPod"
        fontsize: 30
        label_color: "white"
        label_x_offset: 120
        enabled: false
        visible: bluetoothAdapter.connected && main.isTouch
    }

    Item {
        id: bluetooth_controls
        x: 562

        Connections {
            target: main.isTft && !main.blockKeys && root.keysAccepted && bluetoothAdapter.connected ? keymapper : null

            onUpButtonPressed: {
                main.debugKeys("upButtonPressed", "MediaBrowser", "(bluetooth controls)")
                volume_controls.volumeUp()
            }
            onDownButtonPressed: {
                main.debugKeys("downButtonPressed", "MediaBrowser", "(bluetooth controls)")
                volume_controls.volumeDown()
            }

            onEnter2Pressed: {
                main.debugKeys("enter2Pressed", "MediaBrowser", "(bluetooth controls)")
                bt_center_button.clicked()
            }
            onDoubleLeftPressed: {
                main.debugKeys("doubleLeftPressed", "MediaBrowser", "(bluetooth controls)")
                bt_left_button.clicked()
            }
            onDoubleRightPressed: {
                main.debugKeys("doubleRightPressed", "MediaBrowser", "(bluetooth controls)")
                bt_right_button.clicked()
            }
        }

        visible: bluetoothAdapter.connected && main.isTouch
        width: 300
        height: 300
        anchors.top: parent.top
        anchors.topMargin: 86
        ImageButton {
            id: bt_up_button
            x: 43
            y: -93
            anchors.bottom: bt_center_button.verticalCenter
            anchors.bottomMargin: 42
            anchors.horizontalCenterOffset: 0
            anchors.horizontalCenter: bt_center_button.horizontalCenter
            upimage: "graphics/205/IPOD_Wheel_Top_Up.png"
            downimage: "graphics/205/IPOD_Wheel_Top_Down.png"
            icon: "graphics/205/IPOD_Controls_Up.png"
            mouseAreaScale: 0.7
            enabled: false

        }
        ImageButton {
            id: bt_center_button
            x: 114
            y: 66

            upimage: (ipodmanager.playStatus == "playing") ? downimage : "graphics/205/IPOD_Wheel_Center_Up.png"
            downimage: "graphics/205/IPOD_Wheel_Center_Down.png"
            icon: "graphics/205/IPOD_Controls_Select.png"

            onClicked: {
                if(bluetoothAdapter.currentConnectedDevice.isPlaying){
                    bluetoothAdapter.currentConnectedDevice.pause()
                }else{
                    bluetoothAdapter.currentConnectedDevice.play()
                }
            }
            mouseAreaScale: 0.7
        }

        ImageButton {
            id: bt_down_button
            anchors.horizontalCenterOffset: 1
            anchors.top: bt_center_button.verticalCenter
            anchors.topMargin: 42
            anchors.horizontalCenter: bt_center_button.horizontalCenter
            upimage: "graphics/205/IPOD_Wheel_Bottom_Up.png"
            downimage: "graphics/205/IPOD_Wheel_Bottom_Down.png"
            icon: "graphics/205/IPOD_Controls_Down.png"
            mouseAreaScale: 0.7
            enabled: false

        }

        ImageButton {
            id: bt_left_button
            x: 0
            y: 116
            anchors.right: bt_center_button.horizontalCenter
            anchors.rightMargin: 36
            anchors.verticalCenterOffset: 0
            anchors.verticalCenter: bt_center_button.verticalCenter
            anchors.topMargin: 0
            upimage: "graphics/205/IPOD_Wheel_Left_Up.png"
            downimage: "graphics/205/IPOD_Wheel_Left_Down.png"
            icon: "graphics/205/IPOD_Controls_Back.png"
            onClicked: {
                if (bluetoothAdapter.connected) {
                    bluetoothAdapter.currentConnectedDevice.previous();
                }
            }
            mouseAreaScale: 0.7

        }

        ImageButton {
            id: bt_right_button
            anchors.verticalCenterOffset: 0
            anchors.left: bt_center_button.horizontalCenter
            anchors.leftMargin: 41
            anchors.verticalCenter: bt_center_button.verticalCenter
            upimage: "graphics/205/IPOD_Wheel_Right_Up.png"
            downimage: "graphics/205/IPOD_Wheel_Right_Down.png"
            icon: "graphics/205/IPOD_Controls_Forward.png"
            onClicked: {
                bluetoothAdapter.currentConnectedDevice.next();
            }
            mouseAreaScale: 0.7

        }

        Rectangle {
            id: bt_wheel_well
            // helpful for alignment of circular button array, as buttons have shadows
            x: 75
            y: 43
            width: 285
            height: width
            color: "#ffffff"
            radius: width*0.5
            anchors.verticalCenterOffset: -2
            anchors.horizontalCenterOffset: -2
            z: -1
            anchors.horizontalCenter: bt_center_button.horizontalCenter
            anchors.verticalCenter: bt_center_button.verticalCenter
            opacity: 0
        }
    } // end ipod wheel bluetooth


    // Help pane for Bluetooth TFT
    BluetoothTftHelp {
        id: bt_help_pane
        keysAccepted: root.keysAccepted
        visible: bluetoothAdapter.connected && main.isTft
        x: 532
        anchors.top: parent.top
    }

    Connections{
        target: bluetoothAdapter
        onPasskeyAvailable:{
            var text = passkey.toString()
            btStatusDialog.state = "pairing"
            btPairCodeKeyText.text = paddy(text,6)
        }
        onDiscoveringChanged: {
            if(discovering && bluetoothAdapter.nonLeScan){
                btStatusDialog.state = "list"
            }
        }

        onPairingComplete:{
            if(result) {
                btStatusDialog.state = "connecting"
            } else {
                btStatusDialog.state = "idle"
            }
        }

        onConnectionFailed: {
            btStatusDialog.state = "idle"
        }

        onConnectedChanged:{
            if(connected){
                btStatusDialog.state = "idle"
            }else{
                btStatusDialog.state = "disconnecting"
            }
        }

        onDisconnecting:{
            console.debug("disconnecting signal received")
            btStatusDialog.state = "disconnecting"
        }

        onDisconnectingComplete:{
            console.debug("disconnectingComplete signal received")
            btStatusDialog.state = "idle"
        }

        onPairingCancelling:{
            btStatusDialog.state = "disconnecting"
        }

        onPairingCancelled:{
            btStatusDialog.state = "idle"
        }
    }

    ConfirmationPromptTft {
        id: confirmDisconnectDialog
        isTop: false
        visible: false
        anchors.centerIn: parent
        prompt: qsTr("This will disconnect the device. Are you sure?") + application.emptyString
        onYesClicked: {
            if(bluetoothAdapter.connected) {
                bluetoothAdapter.disconnect()
            }
            confirmDisconnectDialog.hide()
        }
        onNoClicked: {
            confirmDisconnectDialog.hide()
        }
    }

    ModalDialog {
        id: btStatusDialog
        isTop: false
        visible: false
        property string text: "test"
        state: "idle"
        opacity: main.isTouch || root.keysAccepted ? 1.0 : 0.5

        onStateChanged: {
            if(state == "idle"){
                if(bluetoothAdapter.discovering){
                    bluetoothAdapter.stopDiscovery()
                }
            }
        }

        anchors.verticalCenter: parent.verticalCenter
        anchors.horizontalCenter: parent.horizontalCenter

        Connections {
            target: main.isTft && !main.blockKeys && root.keysAccepted && btStatusDialog.visible && !btHelpDialog.visible ? keymapper : null

            onDoubleLeftPressed: {
                main.debugKeys("doubleLeftPressed", "MediaBrowser", "(btStatusDialog)")
                btExitButton.clicked()
            }
        }

        BorderImage {   // pop-up dialog background
            id: btStatusDialogborder

            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            width: root.width * 0.8
            height: width / 2
            source: "../graphics/Pop_Up_Lrg_Grey.png"
            border { top: height*0.33; bottom: height*0.33; left: width*0.33; right: width*0.33 }

            Text {
                id: btScanningText
                visible: false
                anchors.top: parent.top
                anchors.topMargin: 50
                anchors.horizontalCenter: parent.horizontalCenter
                width: parent.width * 0.8
                //height: parent.height * 0.5
                color: "white"
                text: qsTr("SCANNING FOR BLUETOOTH DEVICES...") + application.emptyString
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.bold: true
                font.family: "sans serif"
                font.pixelSize: parent.height * 0.10
            }

            Item {
                id: scanningAnimation
                height: 58
                width: 108
                anchors.top: btScanningText.bottom
                anchors.topMargin: 50
                anchors.horizontalCenter: parent.horizontalCenter
                Image{
                    id: scanningBluetoothWave
                    property int opacityInteger: 0
                    anchors.fill: parent
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.horizontalCenter: parent.horizontalCenter
                    source: "../graphics/Bluetooth/Bluetooth_waves.png"
                    opacity: opacityInteger / 100.0
                    NumberAnimation on opacityInteger {
                        from: 0;
                        to: 100;
                        duration: 500
                        running : scanningAnimation.visible
                        loops: Animation.Infinite
                    }
                }
                Image{
                    id: scanningBluetoothIcon
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.horizontalCenter: parent.horizontalCenter
                    source: "../graphics/Bluetooth/Bluetooth_icon_medium.png"

                }
            }

            Item {
                id: btPairingItems
                visible: false
                anchors.verticalCenter: parent.verticalCenter
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.fill: parent
                Text {
                    id: btPairCodeText
                    visible: parent.visible
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.verticalCenterOffset: -implicitHeight / 2
                    width: parent.width * 0.50

                    color: "white"
                    text: qsTr("CONFIRM ON DEVICE:") + application.emptyString
                    horizontalAlignment: Text.AlignHCenter
                    wrapMode: Text.WordWrap
                    font.bold: true
                    font.family: "sans serif"
                    font.pixelSize: parent.height * 0.14
                }
                Text {
                    id: btPairCodeKeyText
                    visible: parent.visible
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.verticalCenterOffset: implicitHeight / 2
                    width: parent.width * 0.50
                    color: "white"
                    text: ""
                    horizontalAlignment: Text.AlignHCenter
                    wrapMode: Text.WordWrap
                    font.bold: true
                    font.family: "sans serif"
                    font.pixelSize: parent.height * 0.14
                }
            }



            Item{
                id: btDeviceListBox
                visible: false
                anchors.verticalCenter: parent.verticalCenter
                anchors.horizontalCenter: parent.horizontalCenter
                width: parent.width * 0.8
                height: parent.height * 0.5

                Text{
                    id: btSelectDeviceText
                    text: qsTr("SELECT YOUR DEVICE:") + application.emptyString
                    width: parent.width * 0.9
                    wrapMode: Text.WordWrap
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignVCenter
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.bottom: bluetoothDeviceListBoxBrowser.top
                    anchors.bottomMargin: 5

                    color: "white"
                    font.bold: true
                    font.family: "sans serif"
                    font.pixelSize: parent.parent.height * 0.13
                }

                Item {
                    id: scanningAnimation2
                    height: 58
                    width: 108
                    anchors.right: bluetoothDeviceListBoxBrowser.left
                    anchors.rightMargin: 50
                    anchors.verticalCenter: bluetoothDeviceListBoxBrowser.verticalCenter
                    Image{
                        id: scanningBluetoothWave2
                        property int opacityInteger: 0
                        anchors.fill: parent
                        anchors.verticalCenter: parent.verticalCenter
                        anchors.horizontalCenter: parent.horizontalCenter
                        source: "../graphics/Bluetooth/Bluetooth_waves.png"
                        opacity: opacityInteger / 100.0
                        visible: waveAnimation.running
                        NumberAnimation on opacityInteger {
                            id: waveAnimation
                            from: 0;
                            to: 100;
                            duration: 500
                            running : bluetoothAdapter.discovering
                            loops: Animation.Infinite
                        }
                    }
                    ImageButton{
                        id: scanningBluetoothIcon2
                        anchors.verticalCenter: parent.verticalCenter
                        anchors.horizontalCenter: parent.horizontalCenter
                        upimage: "graphics/Bluetooth/Bluetooth_icon_medium.png"
                        downimage: upimage
                        onClicked: if(!bluetoothAdapter.discovering) bluetoothAdapter.startDiscovery()
                    }
                }

                Rectangle {
                    id: bluetoothDeviceListBoxBrowser

                    color: "transparent"
                    radius: 10
                    width: parent.width * 0.5
                    height: parent.height * 0.7
                    anchors.bottom: parent.bottom
                    anchors.horizontalCenter: parent.horizontalCenter

                    //visible: !bluetoothAdapter.discovering && !bluetoothAdapter.connected && bluetoothDeviceList.model.count

                    DeviceListModel {
                        id: deviceListModel
                    }

                    ListView {
                        id: bluetoothDeviceList

                        Connections {
                            target: main.isTft && !main.blockKeys && root.keysAccepted && btStatusDialog.visible && btStatusDialog.state === "list" && !btHelpDialog.visible ? keymapper : null

                            onUpButtonPressed: {
                                main.debugKeys("upButtonPressed", "MediaBrowser", "(bluetoothDeviceList)")
                                bluetoothDeviceList.decrementCurrentIndex()
                            }

                            onDownButtonPressed: {
                                main.debugKeys("downButtonPressed", "MediaBrowser", "(bluetoothDeviceList)")
                                bluetoothDeviceList.incrementCurrentIndex()
                            }

                            onEnter2Pressed: {
                                main.debugKeys("enter2Pressed", "MediaBrowser", "(bluetoothDeviceList)")
                                bluetoothDeviceList.currentItem.pressed()
                            }
                        }

                        clip: true
                        anchors.fill: parent
                        anchors.margins: 5
                        model: deviceListModel.model
                        highlight: Rectangle { color: "#5f9e45"; radius: 0 ; opacity: 1.0; width: 400 }
                        highlightMoveSpeed: 1500
                        delegate: Component {
                            Item{
                                id: listItem
                                function pressed() {
                                    var item;
                                    item = bluetoothDeviceList.model.getItem(index);
                                    if (item.dummy) {
                                        switch(item.action){
                                            case "refresh":
                                                if(!bluetoothAdapter.discovering) bluetoothAdapter.startDiscovery()
                                                break
                                            case "help":
                                                btHelpDialog.show()
                                                btHelpDialog.startTimeout(10)
                                                break
                                        }
                                    } else {
                                        btStatusDialog.state = "gettingpairingcode"
                                        bluetoothAdapter.connect(item)
                                    }
                                    bluetoothDeviceList.currentIndex = index
                                    brainboard.buzzerButtonPress()
                                }
                                height: 40
                                width: parent.width
                                Text {
                                    text: name
                                    color: "white"
                                    font.bold: true
                                    font.family: "sans serif"
                                    font.pixelSize: parent.height * 0.45
                                    verticalAlignment: Text.AlignVCenter
                                    anchors.left: parent.left
                                    anchors.leftMargin: 10
                                    anchors.verticalCenter: parent.verticalCenter
                                    MouseArea {
                                        anchors.fill: parent
                                        onClicked: {
                                            listItem.pressed(index)
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                Column {
                    id: btDeviceListArrows

                    width: 40
                    height: 160
                    spacing: 50
                    visible: bluetoothDeviceList.model.count

                    anchors {   left: bluetoothDeviceListBoxBrowser.right;
                                verticalCenter: bluetoothDeviceListBoxBrowser.verticalCenter
                    }
                    Image {
                        id: dataUpArrow
                        source: "../graphics/Swipe_Arrows_Left.png"
                        rotation: 90
                    }
                    Image {
                        id: dataDownArrow
                        source: "../graphics/Swipe_Arrows_Left.png"
                        rotation: -90
                    }
                }
            }

            Text {
                id: btGettingPairingCodeText
                visible: false
                anchors.verticalCenter: parent.verticalCenter
                anchors.horizontalCenter: parent.horizontalCenter
                width: parent.width * 0.8
                height: parent.height * 0.5
                color: "white"
                text: qsTr("GETTING PAIRING CODE...") + application.emptyString
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.bold: true
                font.family: "sans serif"
                font.pixelSize: parent.height * 0.15
            }


            Text {
                id: btConnectingText
                visible: false
                anchors.verticalCenter: parent.verticalCenter
                anchors.horizontalCenter: parent.horizontalCenter
                width: parent.width * 0.8
                height: parent.height * 0.5
                color: "white"
                text: qsTr("CONNECTING TO BLUETOOTH DEVICE...") + application.emptyString
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.bold: true
                font.family: "sans serif"
                font.pixelSize: parent.height * 0.15
            }

            Text {
                id: btDisconnectingText
                visible: false
                anchors.verticalCenter: parent.verticalCenter
                anchors.horizontalCenter: parent.horizontalCenter
                width: parent.width * 0.8
                height: parent.height * 0.5
                color: "white"
                text: qsTr("DISCONNECTING FROM BLUETOOTH DEVICE...") + application.emptyString
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.bold: true
                font.family: "sans serif"
                font.pixelSize: parent.height * 0.15
            }

            ImageButton{
                id: btExitButton
                visible: false
                upimage: "Keypad/KeyPad_X.png"
                downimage: "Keypad/KeyPad_X.png"
                anchors.top: parent.top
                anchors.topMargin: 70
                anchors.right: parent.right
                anchors.rightMargin: 70
                onClicked: {
                    if(bluetoothAdapter.pairing == true){
                        bluetoothAdapter.cancelDeviceCreation()
                    } else {
                        btStatusDialog.state = "idle"
                    }
                }
            }

        }

        states: [
            State{
                name: "idle"
                PropertyChanges {target: btStatusDialog; visible: false}
                PropertyChanges {target: btScanningText; visible: false}
                PropertyChanges {target: btPairingItems; visible: false}
                PropertyChanges {target: btDeviceListBox; visible: false}
                PropertyChanges {target: btConnectingText; visible: false}
                PropertyChanges {target: btExitButton; visible: false}
                PropertyChanges {target: scanningAnimation; visible: false}
                PropertyChanges {target: btDisconnectingText; visible: false}
                PropertyChanges {target: btGettingPairingCodeText; visible: false}

            },
            State{
                name: "scanning"
                PropertyChanges {target: btStatusDialog; visible: true}
                PropertyChanges {target: btScanningText; visible: true}
                PropertyChanges {target: btPairingItems; visible: false}
                PropertyChanges {target: btDeviceListBox; visible: false}
                PropertyChanges {target: btConnectingText; visible: false}
                PropertyChanges {target: btExitButton; visible: false}
                PropertyChanges {target: scanningAnimation; visible: true}
                PropertyChanges {target: btDisconnectingText; visible: false}
                PropertyChanges {target: btGettingPairingCodeText; visible: false}
            },
            State{
                name: "list"
                PropertyChanges {target: btStatusDialog; visible: true}
                PropertyChanges {target: btScanningText; visible: false}
                PropertyChanges {target: btPairingItems; visible: false}
                PropertyChanges {target: btDeviceListBox; visible: true}
                PropertyChanges {target: btConnectingText; visible: false}
                PropertyChanges {target: btExitButton; visible: main.isTouch}
                PropertyChanges {target: scanningAnimation; visible: false}
                PropertyChanges {target: btDisconnectingText; visible: false}
                PropertyChanges {target: btGettingPairingCodeText; visible: false}
            },
            State{
                name: "pairing"
                PropertyChanges {target: btStatusDialog; visible: true}
                PropertyChanges {target: btScanningText; visible: false}
                PropertyChanges {target: btPairingItems; visible: true}
                PropertyChanges {target: btDeviceListBox; visible: false}
                PropertyChanges {target: btConnectingText; visible: false}
                PropertyChanges {target: btExitButton; visible: main.isTouch}
                PropertyChanges {target: scanningAnimation; visible: false}
                PropertyChanges {target: btDisconnectingText; visible: false}
                PropertyChanges {target: btGettingPairingCodeText; visible: false}
            },
            State{
                name: "connecting"
                PropertyChanges {target: btStatusDialog; visible: true}
                PropertyChanges {target: btScanningText; visible: false}
                PropertyChanges {target: btPairingItems; visible: false}
                PropertyChanges {target: btDeviceListBox; visible: false}
                PropertyChanges {target: btConnectingText; visible: true}
                PropertyChanges {target: btExitButton; visible: main.isTouch}
                PropertyChanges {target: btDisconnectingText; visible: false}
                PropertyChanges {target: btGettingPairingCodeText; visible: false}
            },
            State{
                name: "disconnecting"
                PropertyChanges {target: btStatusDialog; visible: true}
                PropertyChanges {target: btScanningText; visible: false}
                PropertyChanges {target: btPairingItems; visible: false}
                PropertyChanges {target: btDeviceListBox; visible: false}
                PropertyChanges {target: btConnectingText; visible: false}
                PropertyChanges {target: btExitButton; visible: false}
                PropertyChanges {target: btDisconnectingText; visible: true}
                PropertyChanges {target: btGettingPairingCodeText; visible: false}
            },
            State{
                name: "gettingpairingcode"
                PropertyChanges {target: btStatusDialog; visible: true}
                PropertyChanges {target: btScanningText; visible: false}
                PropertyChanges {target: btPairingItems; visible: false}
                PropertyChanges {target: btDeviceListBox; visible: false}
                PropertyChanges {target: btConnectingText; visible: false}
                PropertyChanges {target: btExitButton; visible: false}
                PropertyChanges {target: btDisconnectingText; visible: false}
                PropertyChanges {target: btGettingPairingCodeText; visible: true}
            }
        ]
    }

    function paddy(n, p, c){
        var pad_char = typeof c !== 'undefined' ? c : '0';
        var pad = new Array(1+p).join(pad_char);
        return (pad+n).slice(-pad.length);
    }

}

