import QtQuick 1.1
import QtWebKit 1.0

import ".."
import "../Keypad"

Item{
    id: root

    property string url
    property bool keyingURL: false
    property string defaultURL: options.homeURL
    property bool noSwipe: true
    property int currentContentWidth:0

    function viewdestruct() {
        console.log("viewdestruct called! (WebViewContainer.qml)")
        root.url = ""
        root.clearCookies()
    }

    function clearCookies() {
        nami.clearCookies()
        //webView.reload.trigger()
    }

    function keypress(val) {
        if(root.keyingURL)
            return
        if(val === "ENTER")
            application.emitKeypress(Qt.Key_Enter)
        else if(val === "<<")
            application.emitKeypress(Qt.Key_Backspace)
        else
            application.emitKeypress(val)
    }

    function shown() {
        print("current URL:" + webView.url)
        if(!root.url && root.defaultURL) {
            root.url = root.defaultURL
        }
        if(!webView.url) {
            urlKeypad.show()
        }
        webView.renderingEnabled = true
        if (flashWatcher.soundIsPlaying)
            audioManager.flashRequest()
    }

    function hidden() {
        urlKeypad.hide()
        inputKeypad.hide()
        webView.renderingEnabled = false
    }

    onUrlChanged: {
        webView.url = webView.fixUrl(url)
    }

    width:workoutViewContainer.contentAreaWidth
    height: workoutViewContainer.contentAreaHeight

    AlphaKeypad {
        id: urlKeypad
        show_entry: true
        anchors { horizontalCenter: parent.horizontalCenter ; bottom: parent.bottom }
        visible: false
        z: 9999
        onClose: hide()

        onValueEntered: {
            root.url = val
            urlKeypad.hide()
            urlKeypad.clear()
        }
    }

    AlphaKeypad {
        id: inputKeypad
        show_entry: false
        anchors { horizontalCenter: parent.horizontalCenter ; bottom: parent.bottom }
        visible: false
        z: 9999
        onClose: hide()

        onKeyPress: root.keypress(key)
    }

    Rectangle {
        id: buttonBar

        color: "darkgray"
        radius: 5
        anchors { top: parent.top ; left: parent.left ; right: parent.right }
        height: 40

        Row {
            id: buttonRow
            spacing: 15

            ImageButton {
                id: backButton

                upimage: "graphics/webview/web_leftarrow.png"
                downimage: "graphics/webview/web_leftarrow_DOWN.png"
                enabled: webView.back.enabled
                anchors { top: parent.top ; topMargin: 2}

                onClicked: webView.back.trigger()
            }

            ImageButton {
                id: forwardButton

                upimage: "graphics/webview/web_rightarrow.png"
                downimage: "graphics/webview/web_rightarrow_DOWN.png"
                enabled: webView.forward.enabled
                anchors { top: parent.top ; topMargin: 2 }

                onClicked: webView.forward.trigger()
            }

            ImageButton {
                id: keyboardButton

                upimage: "graphics/webview/web_keyboard.png"
                downimage: "graphics/webview/web_keyboard_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }

                onClicked: {
                    inputKeypad.clear()
                    if (inputKeypad.visible)
                        inputKeypad.hide()
                    else {
                        urlKeypad.hide()
                        inputKeypad.show()
                    }
                }
            }

            ImageButton {
                id: homeButton

                visible: options.homeURL
                upimage: "graphics/webview/web_home.png"
                downimage: "graphics/webview/web_home_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }
                onClicked: root.url = root.defaultURL
            }

            ImageButton {
                id: linksButton

                visible: options.linksURL
                upimage: "graphics/webview/web_bookmarks.png"
                downimage: "graphics/webview/web_bookmarks_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }
                onClicked: root.url = options.linksURL
            }
        }
        Rectangle {
            id: urlBar

            color: "white"
            radius: 5
            height: 32
            anchors { left: buttonRow.right ; leftMargin: 15 ;  right: parent.right ; rightMargin: 8 ; top: parent.top; topMargin: 3 }

            Rectangle {
                id: progressBar

                color: "lightblue"
                opacity: 0.6
                radius: 5
                width: (webView.progress < 1) ? webView.progress* urlBar.width : 0
                anchors { left: urlBar.left ; top: urlBar.top ; bottom: urlBar.bottom }
            }

            Text {
                id: urlEntry

                text: urlKeypad.visible ? urlKeypad.value : webView.url
                font.pointSize: 12
                anchors { verticalCenter: parent.verticalCenter ; left: parent.left ; leftMargin: 5 }
                width: (paintedWidth < parent.width) ? paintedWidth : parent.width
                clip: true
                height: paintedHeight
            }

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    urlKeypad.setValue(webView.url)
                    inputKeypad.close()
                    urlKeypad.show()
                }
            }
        }
    }

    Flickable {
        id: flickable

        anchors { top: buttonBar.bottom ; bottom: parent.bottom ; left: parent.left ; right: parent.right }
        contentWidth: Math.max(1000, webView.width)
        contentHeight: Math.max(406, webView.height)
        smooth: false
        clip: true

        WebView {
            id: webView

            settings.pluginsEnabled: true
            settings.javascriptCanOpenWindows: false

//            onWidthChanged: console.log("webview width",width)
//            onHeightChanged: console.log("webview height",height)

            function fixUrl(url) {
                if(url=="") return url
                if(url[0] == "/") return "file://" + url
                if(url.indexOf(":") < 0) {
                    if(url.indexOf(".") < 0 || url.indexOf(" ") >= 0){
                        print("url fallback ")
                        if(root.defaultURL)
                            return root.defaultURL
                        else
                            return "http://www.truefitness.com"
                    }
                    else
                        return "http://" + url
                }
                else
                    return url
            }

            function doZoom(zoom, centerX, centerY) {
                console.log("doZoom called",zoom,centerX,centerY)
                if(centerX) {
                    var sc = zoom* contentsScale
                    scaleAnim.to = sc
                    flickVX.from = flickable.contentX
                    flickVX.to = Math.max(0, Math.min(centerX - flickable.width/2, webView.width*sc-flickable.width))
                    finalX.value = flickVX.to
                    flickVY.from = flickable.contentY
                    flickVY.to = Math.max(0, Math.min(centerY-flickable.height/2, webView.height*sc-flickable.height))
                    finalY.value = flickVY.to
                    quickZoom.start()
                }
            }

            onLoadFailed: {
                if (webView.progress < 1.0) {
                    console.log("Failed to load webpage", root.url, webView.progress)
                    loadFailedPopup.showWithTimeout(3)
                }
            }

            onDoubleClick: {
                if(!heuristicZoom(clickX, clickY, 2.5)) {
                    var zf = flickable.width / contentsSize.width
                    if( zf >= contentsScale)
                        zf = 2.0*contentsScale
                    doZoom(zf, clickX*zf, clickY*zf)
                }else console.log("doing heuristicZoom");
            }
            contentsScale: 1
            onContentsSizeChanged: {
                if(Math.abs(contentsSize.width - currentContentWidth) > 20 ){
                    contentsScale = Math.min(1, flickable.width / (contentsSize.width / contentsScale))
                    currentContentWidth = contentsSize.width
                    console.log("onContentsSizeChanged", contentsScale, flickable.width, contentsSize.width, currentContentWidth, progress)
                }
            }
            onZoomTo: doZoom(zoom, centerX, centerY)
            onUrlChanged: {
                flickable.contentX = 0
                flickable.contentY = 0
                if(url != root.url)
                    root.url = url
            }

            transformOrigin: Item.TopLeft
            preferredWidth: 1000
            preferredHeight: 406
            anchors.bottom: parent.bottom
            anchors.horizontalCenter: parent.horizontalCenter
            smooth: false
        }
    }

    MessagePopup {
        id: loadFailedPopup
        message: qsTr("The Requested Page Could Not Be Loaded") + application.emptyString
        xOffset: 332
        yOffset: 222
    }

    ImageButton {
        id: topButton

        upimage: "graphics/webview/web_leftarrow.png"
        downimage: "graphics/webview/web_leftarrow_DOWN.png"
        visible: flickable.contentY > 25
        opacity: 0.5
        rotation: 90
        anchors { top: flickable.top  ; right: flickable.right }

        onClicked: scrollToTop.start()

        NumberAnimation {
            id: scrollToTop
            target: flickable
            property: "contentY"
            easing.type: Easing.Linear
            duration: 500
            to: 0
        }
    }

    Rectangle {
        id: scrollPos

        color: "gray"
        opacity: 0.5
        width: 10
        height: flickable.height * ( flickable.height / webView.contentsSize.height)

        anchors { right: flickable.right ; top: flickable.top ; topMargin: flickable.contentY * ( flickable.height / flickable.contentHeight)}
    }

    SequentialAnimation {
        id: quickZoom

        PropertyAction { target: webView ; property: "renderingEnabled" ; value: false }
        ParallelAnimation {
            NumberAnimation {
                id: scaleAnim
                target: webView
                property: "contentsScale"
                easing.type: Easing.Linear
                duration: 200
            }
            NumberAnimation {
                id: flickVX
                target: flickable
                property: "contentX"
                easing.type: Easing.Linear
                duration: 200
                from: 0
                to: 0
            }
            NumberAnimation {
                id: flickVY
                target: flickable
                property: "contentY"
                easing.type: Easing.Linear
                duration: 200
                from: 0
                to: 0
            }
        } // parallel animation
        PropertyAction { id: finalX ; target: flickable ; property: "contentX" ; value: 0 }
        PropertyAction { id: finalY ; target: flickable ; property: "contentX" ; value: 0 }
        PropertyAction { target: webView ; property: "renderingEnabled" ; value: true }
    }
}
